package eu.himeros.transcoder;

import java.io.BufferedWriter;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.Stack;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 *
 * @author federico.boschetti
 */
public class SaxTranscoder extends TransHandler {
    Stack<String> stk=new Stack<String>();
    XMLReader xr = null;
    Transcoder trans = null;
    String triggerTag=null;
    String triggerAttr=null;
    String triggerVal=null;
    boolean triggered=false;
    String flagStr="#";
    //SaxTranscoder handler=null;

    public SaxTranscoder() {
        super();
        //writeStartDocument=true;
        //writeProcessingInstruction=true;
        //writeIgnorableWhiteSpaces=true;
        //writeComment=true;
        //writeStartCDATA=true;
        //writeEndCDATA=true;
    }

    public void parse(String inFile, String outFile, String transFile, boolean reverseBool, String trigger) {
        try {
            if(trigger.contains("=")){
                triggerAttr=trigger.substring(0,trigger.indexOf('='));
                triggerVal=trigger.substring(trigger.indexOf('=')+1,trigger.length());
                triggerVal=triggerVal.replaceAll("\\'|\\\"","");
            }else{
                triggerTag=trigger;
            }
            bw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(outFile), "UTF-8"));
            trans = new Transcoder(transFile);
            trans.reverse(reverseBool);
            xr = XMLReaderFactory.createXMLReader();
            xr.setProperty("http://xml.org/sax/properties/lexical-handler",this);
            xr.setContentHandler(this);

            xr.setErrorHandler(this);
            xr.parse(inFile);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void characters(char[] ch, int start, int length) {
        String s = new String(ch, start, length);
        //if(s.endsWith("]")) System.out.println(s);
        try {
            if(triggered){
                bw.write(trans.parse(s));
            }else{
                bw.write(s);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void startDocument() throws SAXException{
        super.startDocument();
    }

    public void endDocument() {
        try {
            bw.flush();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
        super.startElement(uri, localName, qName, attributes);
        if(triggerTag!=null){
            if(qName.equals(triggerTag)){
                flagStr="*";
                triggered=true;
            }
        }else if(attributes!=null&&attributes.getValue(triggerAttr)!=null){
                triggered=attributes.getValue(triggerAttr).equals(triggerVal);
                if(triggered){
                    flagStr="*";
                }else{
                    flagStr="#";
                }
        }else{
            if(!stk.empty()){
                flagStr=stk.peek();
                if(flagStr.equals("*")) triggered=true; else triggered=false;
            }
        }
        stk.push(flagStr);
        try {
            bw.write(sb.toString());
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void endElement(String uri, String localName, String qName) throws SAXException {
        super.endElement(uri, localName, qName);
        if(!stk.empty()){
            flagStr=stk.pop();
        }
        if(!stk.empty()&&stk.peek().equals("*")){
            triggered=true;
        }else{
            triggered=false;
        }
        try {
            bw.write(sb.toString());
        } catch (IOException ex) {
            Logger.getLogger(SaxTranscoder.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        if(args==null||args.length<5){
            System.out.println("Usage: eu.himeros.transcoder.SaxTranscoder <inFile> <outFile> <transFile> <reverseBool> <triggerTag||triggerAttr=Val>");
        }
        // TODO code application logic here
        SaxTranscoder saxTrans = new SaxTranscoder();
        saxTrans.parse(args[0], args[1], args[2],Boolean.parseBoolean(args[3]),args[4]);
    }
}
