/**
 * @author federico.boschetti.73[AT]gmail.com
 */

package eu.himeros.transcoder;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class Tcoder {

    /**
     * General purpose transcoder
     */

    private static Hashtable<String, String> directHt;
    private static Hashtable<String, String> reverseHt;
    private static Hashtable<String, String> ht;
    private static int maxLen = 0;
    private static Pattern pattern=null;
    private static boolean notInited=true; // not yet initialized

    public static void reverse(boolean reverse){
        if(reverse){
            ht=reverseHt;
        }else{
            ht=directHt;
        }
    }

    public static void reverse(){
        ht=reverseHt;
    }
    
    public static void setTranscoder(String transFileName){
        try {
            /* read code file, transform utf8s in chars and put
            into a hashtable the sequences of char codes */
            String codeFrom;
            String codeTo;
            File transFile = new File(transFileName);
            BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(transFile), "UTF-8"));
            directHt = new Hashtable<String, String>();
            reverseHt = new Hashtable<String, String>();
            String line=null;
            String lineLeft = "";
            String lineRight = "";
            String[] items=null;
            while ((line = br.readLine()) != null) {
                if (!line.startsWith("//")) {
                    items=line.split("\t");
                    lineLeft=items[0];
                    lineRight=items[1];
                    if (lineLeft.contains("\\u")) {
                        codeFrom = utf8sToChars(lineLeft);
                    } else {
                        codeFrom = lineLeft;
                    }
                    if (codeFrom.length() > maxLen) {
                        maxLen = codeFrom.length();
                    }
                    if(lineRight.contains("\\u")){
                        codeTo = utf8sToChars(lineRight);
                    }else{
                        codeTo=lineRight;
                    }
                    directHt.put(codeFrom, codeTo);
                    reverseHt.put(codeTo, codeFrom);
                }
            }
            ht=directHt;
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void setTranscoder(String transFileName, boolean reverse){
            setTranscoder(transFileName);
            reverse(reverse);
        }

    public static void setTranscoder(Hashtable<String,String> ht){
        directHt=ht;
        makeReverseHt();
    }

    public static Hashtable<String,String> getTranscoder(){
        return ht;
    }

    public static Hashtable<String,String> getDirectTranscoder(){
        return directHt;
    }

    public static Hashtable<String,String> getReverseTranscoder(){
        return reverseHt;
    }

    public static String getCodes() {
        String res="";
        Set<String> keys=ht.keySet();
        String key="";
        for(Iterator<String> i=keys.iterator();i.hasNext();){
            key=i.next();
            res+=key+"\t"+ht.get(key)+"\n";
        }
        return res;
    }

    public static String decode(String key){
        return ht.get(key);
    }

    private static String utf8sToChars(String str) {
        if(str.equals("\\uxxxx")) return "";
        StringBuffer result =new StringBuffer();
        Pattern p=Pattern.compile("(?:\\\\)u([0-9A-Fa-f]{4})");
        Matcher m=p.matcher(str);
        while(m.find()){
            m.appendReplacement(result,""+((char)Long.parseLong(m.group(1),16)));
        }
        m.appendTail(result);
        return result.toString();
    }
    
    public static String parse(String inStr){
        if(inStr==null||inStr.length()==0) return "";
        inStr += addSpaces(maxLen);
        String frag = "";
        String outStr = "";
        String code = "";
        int len = inStr.length();
        int iLeft = 0;
        int iRight = maxLen;
        while(iRight <= len){
            while(iRight > iLeft){
                frag = inStr.substring(iLeft, iRight);
                code = ht.get(frag);
                if (code != null) {
                    outStr += code;
                    break;
                }
                iRight--;
            }
            if(iRight==iLeft){
                iRight+=1;
                outStr+=frag;
            }
            iLeft = iRight;
            iRight += maxLen;
        }
        if(outStr.length()>1){
            return outStr.substring(0,outStr.length()-1);
        }else return outStr;
    }

    public static String parse(String transFileName, String reverseBool, String inStr){
        if(notInited){
            setTranscoder(transFileName, Boolean.parseBoolean(reverseBool));
            notInited=false;
        }
        return parse(inStr);
    }

    private static String addSpaces(int n) {
        String res = "";
        for (int i = 0; i < n; i++) {
            res += " ";
        }
        return res;
    }

    public static String parse(String inStr, String regexp){
        pattern=Pattern.compile(regexp);
        Matcher matcher=pattern.matcher(inStr);
        StringBuffer sb=new StringBuffer();
        while(matcher.find()){
            if(matcher.groupCount()==3){
                matcher.appendReplacement(sb, matcher.group(1)+parse(matcher.group(2)+matcher.group(3)));
            }else{
                matcher.appendReplacement(sb, parse(matcher.group(1)));
            }
        }
        matcher.appendTail(sb);
        return sb.toString();
    }

    public static String parse(String transFileName, String reverseBool, String inStr, String regexp){
        if(notInited){
            setTranscoder(transFileName, Boolean.parseBoolean(reverseBool));
            notInited=false;
        }
        return parse(inStr,regexp);
    }

    public static String parse(String inStr, String startTagRegexp, String endTagRegexp, boolean del){
        if(!del){ //if del is true, tags that open and close the sequence are discarded
            startTagRegexp="("+startTagRegexp+")";
            endTagRegexp="("+endTagRegexp+")";
        }
        return parse(inStr, startTagRegexp+"(.*?)"+endTagRegexp);
    }

    public static String parse(String transFileName, String reverseBool, String inStr, String startTagRegexp, String endTagRegexp, String delBool){
        if(notInited){
            setTranscoder(transFileName, Boolean.parseBoolean(reverseBool));
            notInited=false;
        }
        return parse(inStr, startTagRegexp, endTagRegexp, Boolean.parseBoolean(delBool));
    }

    private static void makeReverseHt(){
        String val=null;
        String key=null;
        Enumeration<String> keys=directHt.keys();
        while(keys.hasMoreElements()){
            key=keys.nextElement();
            val=directHt.get(key);
            reverseHt.put(val, key);
        }
    }
    
    public static void printUsage(){
            System.out.println("usage: eu.himeros.transcoder.Tcoder.parse(<transFile> <reverseBool> <inStr>)");
    }
}