// DefaultHandler.java - default implementation of the core handlers.
// http://www.saxproject.org
// Written by David Megginson
// NO WARRANTY!  This class is in the public domain.
// $Id: DefaultHandler.java,v 1.3 2006/04/13 02:06:32 jeffsuttor Exp $
package eu.himeros.transcoder;

import java.io.BufferedWriter;
import java.io.IOException;

import org.xml.sax.InputSource;
import org.xml.sax.Locator;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.ext.DeclHandler;
import org.xml.sax.ext.DefaultHandler2;
import org.xml.sax.ext.EntityResolver2;
import org.xml.sax.ext.LexicalHandler;

public class TransHandler extends DefaultHandler2 implements LexicalHandler, DeclHandler, EntityResolver2 {

    protected StringBuffer sb = null;
    protected BufferedWriter bw=null;

    protected boolean writeNotationDecl=false;
    protected boolean writeUnparsedEntityDecl=false;
    protected boolean writeSetDocumentLocator=false;
    protected boolean writeStartDocument=false;
    protected boolean writeEndDocument=false;
    protected boolean writeStartPrefixMapping=false;
    protected boolean writeEndPrefixMapping=false;
    protected boolean writeStartElement=false;
    protected boolean writeEndElement=false;
    protected boolean writeCharacters=false;
    protected boolean writeIgnorableWhiteSpaces=false;
    protected boolean writeProcessingInstruction=false;
    protected boolean writeSkippedEntity=false;
    protected boolean writeWarning=false;
    protected boolean writeError=false;
    protected boolean writeFatalError=false;
    protected boolean writeStartCDATA=false;
    protected boolean writeEndCDATA=false;
    protected boolean writeStartDTD=false;
    protected boolean writeEndDTD=false;
    protected boolean writeStartEntity=false;
    protected boolean writeEndEntity=false;
    protected boolean writeComment=false;
    protected boolean writeAttributeDecl=false;
    protected boolean writeElementDecl=false;
    protected boolean writeExternalEntityDecl=false;
    protected boolean writeInternalEntityDecl=false;
    protected boolean writeGetExternalSubset=false;
    protected boolean writeResolveEntity=false;

    public void notationDecl(String name, String publicId, String systemId) throws SAXException {
        sb=new StringBuffer("");
        try{if(writeNotationDecl) bw.write(sb.toString());}catch(Exception ex){ex.printStackTrace();}
    }

    public void unparsedEntityDecl(String name, String publicId, String systemId, String notationName) throws SAXException {
        sb=new StringBuffer("");
        try{if(writeUnparsedEntityDecl) bw.write(sb.toString());}catch(Exception ex){ex.printStackTrace();}
    }

    public void setDocumentLocator(Locator locator) {
        sb=new StringBuffer("");
        try{if(writeSetDocumentLocator) bw.write(sb.toString());}catch(Exception ex){ex.printStackTrace();}
    }

    public void startDocument() throws SAXException {
        sb=new StringBuffer("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
        try{if(true){bw.write(sb.toString());bw.newLine();}}catch(Exception ex){ex.printStackTrace();}
    }

    public void endDocument() throws SAXException {
        sb=new StringBuffer("");
        try{if(writeEndDocument) bw.write(sb.toString());}catch(Exception ex){ex.printStackTrace();}
    }

    public void startPrefixMapping(String prefix, String uri) throws SAXException {
        sb=new StringBuffer("");
        try{if(writeStartPrefixMapping) bw.write(sb.toString());}catch(Exception ex){ex.printStackTrace();}
    }

    public void endPrefixMapping(String prefix) throws SAXException {
        sb=new StringBuffer("");
        try{if(writeEndPrefixMapping) bw.write(sb.toString());}catch(Exception ex){ex.printStackTrace();}
    }

    public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
        sb = new StringBuffer();
        sb.append("<" + qName);
        for (int i = 0; i < attributes.getLength(); i++) {
            sb.append(" " + attributes.getQName(i) + "=\"" + attributes.getValue(i) + "\"");
        }
        sb.append(">");
        try{if(writeStartElement) bw.write(sb.toString());}catch(Exception ex){ex.printStackTrace();}
    }

    public void endElement(String uri, String localName, String qName) throws SAXException {
        sb = new StringBuffer();
        sb.append("</" + qName + ">");
        try{if(writeEndElement) bw.write(sb.toString());}catch(Exception ex){ex.printStackTrace();}
    }

    public void characters(char ch[], int start, int length) throws SAXException {
        sb=new StringBuffer(new String(ch,start,length));
        try{if(writeCharacters) bw.write(sb.toString());}catch(Exception ex){ex.printStackTrace();}
    }

    public void ignorableWhitespace(char ch[], int start, int length) throws SAXException {
        sb = new StringBuffer(new String(ch, start, length));
        try{if(writeStartElement) bw.write(sb.toString());}catch(Exception ex){ex.printStackTrace();}
    }

    public void processingInstruction(String target, String data) throws SAXException {
        sb = new StringBuffer("<?" + target + " " + data + "?>");
        try{if(writeProcessingInstruction) bw.write(sb.toString());}catch(Exception ex){ex.printStackTrace();}
    }

    public void skippedEntity(String name) throws SAXException {
        sb=new StringBuffer("");
        try{if(writeSkippedEntity) bw.write(sb.toString());}catch(Exception ex){ex.printStackTrace();}
    }

    public void startCDATA() throws SAXException {
        sb=new StringBuffer("<![CDATA[");
        try{if(writeStartCDATA) bw.write(sb.toString());}catch(Exception ex){ex.printStackTrace();}
    }

    public void endCDATA() throws SAXException {
        sb=new StringBuffer("]]>");
        try{if(writeEndCDATA) bw.write(sb.toString());}catch(Exception ex){ex.printStackTrace();}
    }

    public void startDTD(String name, String publicId, String systemId) throws SAXException {
        sb=new StringBuffer("");
        try{if(writeStartDTD) bw.write(sb.toString());}catch(Exception ex){ex.printStackTrace();}
    }

    public void endDTD() throws SAXException {
        sb=new StringBuffer("");
        try{if(writeEndDTD) bw.write(sb.toString());}catch(Exception ex){ex.printStackTrace();}
    }

    public void startEntity(String name) throws SAXException {
        sb=new StringBuffer("");
        try{if(writeStartEntity) bw.write(sb.toString());}catch(Exception ex){ex.printStackTrace();}
    }

    public void endEntity(String name) throws SAXException {
        sb=new StringBuffer("");
        try{if(writeEndEntity) bw.write(sb.toString());}catch(Exception ex){ex.printStackTrace();}
    }

    public void comment(char ch[], int start, int length) throws SAXException {
        sb=new StringBuffer("<!--"+new String(ch, start, length)+"-->");
        try{if(writeComment) bw.write(sb.toString());}catch(Exception ex){ex.printStackTrace();}
    }

    // SAX2 ext-1.0 DeclHandler
    public void attributeDecl(String eName, String aName, String type, String mode, String value) throws SAXException {
        sb=new StringBuffer("");
        try{if(writeAttributeDecl) bw.write(sb.toString());}catch(Exception ex){ex.printStackTrace();}
    }

    public void elementDecl(String name, String model) throws SAXException {
        sb=new StringBuffer("");
        try{if(writeElementDecl) bw.write(sb.toString());}catch(Exception ex){ex.printStackTrace();}
    }

    public void externalEntityDecl(String name, String publicId, String systemId) throws SAXException {
        sb=new StringBuffer("");
        try{if(writeExternalEntityDecl) bw.write(sb.toString());}catch(Exception ex){ex.printStackTrace();}
    }

    public void internalEntityDecl(String name, String value) throws SAXException {
        sb=new StringBuffer("");
        try{if(writeInternalEntityDecl) bw.write(sb.toString());}catch(Exception ex){ex.printStackTrace();}
    }

    // SAX2 ext-1.1 EntityResolver
    public InputSource getExternalSubset(String name, String baseURI) throws SAXException, IOException {
        sb=new StringBuffer("");
        try{if(writeGetExternalSubset) bw.write(sb.toString());}catch(Exception ex){ex.printStackTrace();}
        return null;
    }
    
    public InputSource resolveEntity(String name, String publicId, String baseURI, String systemId) throws SAXException, IOException {
        sb=new StringBuffer("");
        try{if(writeResolveEntity) bw.write(sb.toString());}catch(Exception ex){ex.printStackTrace();}
        return null;
    }

    public InputSource resolveEntity(String publicId, String systemId) throws SAXException, IOException {
        sb=new StringBuffer("");
        try{if(writeResolveEntity) bw.write(sb.toString());}catch(Exception ex){ex.printStackTrace();}
        return resolveEntity(null, publicId, null, systemId);
    }

    public void warning(SAXParseException e) throws SAXException {
        sb=new StringBuffer("");
        try{if(writeWarning) bw.write(sb.toString());}catch(Exception ex){ex.printStackTrace();}
    }

    public void error(SAXParseException e) throws SAXException {
        sb=new StringBuffer("");
        try{if(writeError) bw.write(sb.toString());}catch(Exception ex){ex.printStackTrace();}
    }

    public void fatalError(SAXParseException e) throws SAXException {
        sb=new StringBuffer("");
        try{if(writeFatalError) bw.write(sb.toString());}catch(Exception ex){ex.printStackTrace();}
        throw e;
    }

}
